""" Unit tests for core_linked_records_app.rest.pid.views
"""
from unittest import TestCase

import json
from unittest.mock import patch

from core_explore_common_app.components.query import api as query_api
from core_federated_search_app.components.instance import (
    api as instance_api,
)
from core_linked_records_app.components.blob import api as blob_api
from core_linked_records_app.components.data import api as data_api
from core_linked_records_app.rest.pid import views as pid_views
from tests import mocks


class TestRetrieveDataPidGet(TestCase):
    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.GET = dict()

    @patch.object(data_api, "get_pid_for_data")
    def test_data_api_get_pid_for_data_fails_returns_500(self, mock_get_pid_for_data):
        self.mock_request.GET["data_id"] = "mock_data_id"
        mock_get_pid_for_data.side_effect = Exception("mock_get_pid_for_data_exception")

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(data_api, "get_pid_for_data")
    def test_data_api_success_returns_200(self, mock_get_pid_for_data):
        self.mock_request.GET["data_id"] = "mock_data_id"
        mock_get_pid_for_data.return_value = "mock_data_id"

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 200)

    # FIXME cannot import oai_pmh_harvester in INSTALLED_APPS
    # def test_oai_record_api_get_pid_for_data_fails_returns_500(self):
    #     pass
    #
    # def test_oai_record_api_success_returns_200(self):
    #     pass

    @patch.object(instance_api, "get_by_name")
    def test_instance_api_get_by_name_fails_returns_500(self, mock_get_by_name):
        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.side_effect = Exception("mock_get_by_name_exception")

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(pid_views, "oauth2_get_request")
    @patch.object(instance_api, "get_by_name")
    def test_oauth2_get_request_fails_returns_500(
        self, mock_get_by_name, mock_oauth2_get_request
    ):
        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.return_value = mocks.MockInstance()
        mock_oauth2_get_request.side_effect = Exception(
            "mock_oauth2_get_request_exception"
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(pid_views, "oauth2_get_request")
    @patch.object(instance_api, "get_by_name")
    def test_fede_success_returns_200(self, mock_get_by_name, mock_oauth2_get_request):
        self.mock_request.GET["fede_data_id"] = "mock_data_id"
        self.mock_request.GET["fede_origin"] = "mock_origin&param=mock_param"
        mock_get_by_name.return_value = mocks.MockInstance()
        mock_oauth2_get_request.return_value = mocks.MockResponse(
            text=json.dumps({"key": "mock_response_text"})
        )

        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 200)

    def test_incorrect_params_returns_400(self):
        test_view = pid_views.RetrieveDataPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 400)


class TestRetrieveBlobPidGet(TestCase):
    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.GET = dict()

    @patch.object(blob_api, "get_pid_for_blob")
    def test_get_pid_for_blob_fails_returns_500(self, mock_get_pid_for_blob):
        self.mock_request.GET["blob_id"] = "mock_blob_id"
        mock_get_pid_for_blob.side_effect = Exception("mock_get_pid_for_blob_exception")

        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(blob_api, "get_pid_for_blob")
    def test_success_returns_200(self, mock_get_pid_for_blob):
        self.mock_request.GET["blob_id"] = "mock_blob_id"
        mock_get_pid_for_blob.return_value = mocks.MockLocalId()

        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 200)

    def test_incorrect_params_returns_400(self):
        test_view = pid_views.RetrieveBlobPIDView()
        response = test_view.get(self.mock_request)

        self.assertEquals(response.status_code, 400)


class TestRetrieveListPidPost(TestCase):
    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest(POST={"query_id": "mock_query_id"})

    @patch.object(query_api, "get_by_id")
    def test_get_by_id_fails_returns_500(self, mock_get_by_id):
        mock_get_by_id.side_effect = Exception("mock_get_by_id_exception")

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(query_api, "get_by_id")
    def test_data_source_no_pid_url_capabilities_returns_500(self, mock_get_by_id):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[mocks.MockDataSource()]
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(pid_views, "send_get_request")
    @patch.object(query_api, "get_by_id")
    def test_send_get_request_fails_returns_500(
        self, mock_get_by_id, mock_send_get_request
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                mocks.MockDataSource(
                    capabilities={"url_pid": "mock_url_pid"},
                    authentication=mocks.MockAuthentication(type="session"),
                )
            ]
        )
        mock_send_get_request.side_effect = Exception("mock_send_get_request_exception")

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_oauth2_post_request_fails_returns_500(
        self, mock_get_by_id, mock_oauth2_post_request
    ):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                mocks.MockDataSource(
                    capabilities={"url_pid": "mock_url_pid"},
                    authentication=mocks.MockAuthentication(
                        type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ]
        )
        mock_oauth2_post_request.side_effect = Exception(
            "mock_oauth2_post_request_exception"
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(query_api, "get_by_id")
    def test_unknown_auth_returns_500(self, mock_get_by_id):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                mocks.MockDataSource(
                    authentication=mocks.MockAuthentication(type="mock_auth")
                )
            ]
        )

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 500)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_status_200_returns_200(self, mock_get_by_id, mock_oauth2_post_request):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                mocks.MockDataSource(
                    capabilities={"url_pid": "mock_url_pid"},
                    authentication=mocks.MockAuthentication(
                        type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ]
        )
        mock_oauth2_post_request.return_value = mocks.MockResponse(json_data=list())

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 200)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_status_400_returns_400(self, mock_get_by_id, mock_oauth2_post_request):
        mock_get_by_id.return_value = mocks.MockQuery(
            data_sources=[
                mocks.MockDataSource(
                    capabilities={"url_pid": "mock_url_pid"},
                    authentication=mocks.MockAuthentication(
                        type="oauth2",
                        params={"access_token": "mock_access_token"},
                    ),
                )
            ]
        )
        mock_oauth2_post_request.return_value = mocks.MockResponse(status_code=400)

        test_view = pid_views.RetrieveListPIDView()
        response = test_view.post(self.mock_request)

        self.assertEquals(response.status_code, 400)



""" Local resolver Blob API.
"""
from importlib import import_module
from logging import getLogger

from core_linked_records_app import settings
from core_linked_records_app.components.local_id import api as local_id_api
from core_linked_records_app.components.local_id.models import LocalId
from core_linked_records_app.utils.path import get_api_path_from_object
from core_main_app.commons import exceptions
from core_main_app.components.blob.models import Blob

logger = getLogger(__name__)


def get_blob_by_pid(pid, user):
    """Return blob object with the given pid.

    Args:
        pid:
        user:

    Returns:
        Blob object
    """
    try:
        # From the PID url (e.g. https://pid-system.org/prefix/record), retrieve
        # only the prefix and record (e.g. prefix/record) stored in DB.
        pid_internal_name = "/".join(pid.split("/")[-2:])
        local_id_object = local_id_api.get_by_name(pid_internal_name)

        # Ensure the LocalID object refers to a Blob
        assert local_id_object.record_object_class and local_id_object.record_object_id

        # From the local ID object, retrieve record module, separating import path
        # and module name.
        record_object_classpath = local_id_object.record_object_class
        record_object_module_path = ".".join(record_object_classpath.split(".")[:-1])
        api_module_name = record_object_classpath.split(".")[-1]

        # Import record module and call 'get_by_id' function with record ID as
        # parameter.
        module = import_module(record_object_module_path)
        return getattr(module, api_module_name).get_by_id(
            local_id_object.record_object_id, user
        )
    except (AssertionError, exceptions.DoesNotExist):
        error_message = f"PID '{pid}' not assigned to blob"

        logger.error(error_message)
        raise exceptions.DoesNotExist(error_message)
    except Exception as exc:
        error_message = (
            f"An error occurred while looking up blob assigned to PID '{pid}'"
        )

        logger.error(f"{error_message}: {str(exc)}")
        raise exceptions.ApiError(error_message)


def get_pid_for_blob(blob_id):
    """Retrieve PID matching the blob ID provided.

    Args:
        blob_id:

    Returns:
        str - PID of the blob object
    """
    try:
        return local_id_api.get_by_class_and_id(
            record_object_class=get_api_path_from_object(Blob()),
            record_object_id=blob_id,
        )
    except exceptions.DoesNotExist as dne:
        raise exceptions.DoesNotExist(str(dne))
    except Exception as exc:
        error_message = (
            f"An error occurred while looking up PID assigned to blob '{blob_id}'"
        )

        logger.error(f"{error_message}: {str(exc)}")
        raise exceptions.ApiError(error_message)


def set_pid_for_blob(blob_id, blob_pid):
    """Retrieve PID matching the blob ID provided.

    Args:
        blob_id:
        blob_pid:
    """
    try:
        record_name = f"{settings.ID_PROVIDER_PREFIX_BLOB}/{blob_pid.split('/')[-1]}"

        try:
            local_id_object = get_pid_for_blob(blob_id)
        except exceptions.DoesNotExist:
            try:
                local_id_object = local_id_api.get_by_name(record_name)
            except exceptions.DoesNotExist:
                local_id_object = None

        if local_id_object:
            local_id_object.record_name = record_name
            local_id_object.record_object_class = get_api_path_from_object(Blob())
            local_id_object.record_object_id = str(blob_id)
        else:
            local_id_object = LocalId(
                record_name=record_name,
                record_object_class=get_api_path_from_object(Blob()),
                record_object_id=str(blob_id),
            )

        return local_id_api.insert(local_id_object)
    except Exception as exc:
        error_message = (
            f"An error occurred while assigning PID '{blob_pid}' to blob '{blob_id}'"
        )

        logger.error(f"{error_message}: {str(exc)}")
        raise exceptions.ApiError(error_message)



""" Unit tests for core_linked_records_app.utils.providers.local
"""
from unittest import TestCase


class TestLocalIdProviderEncodeToken(TestCase):
    def test_returns_none(self):
        pass


class TestLocalIdProviderIsIdAlreadyUsed(TestCase):
    def test_calls_get_method(self):
        pass

    def test_returns_true_if_successful(self):
        pass

    def test_returns_false_if_failure(self):
        pass


class TestLocalIdProviderGet(TestCase):
    def test_non_existing_record_returns_404(self):
        pass

    def test_existing_record_returns_200(self):
        pass


class TestLocalIdProviderCreate(TestCase):
    def test_record_none_generate_new_record(self):
        pass

    def test_non_unique_record_returns_409(self):
        pass

    def test_successful_creation_returns_201(self):
        pass


class TestLocalIdProviderUpdate(TestCase):
    def test_record_in_response_key(self):
        pass

    def test_message_in_response_key(self):
        pass

    def test_url_in_response_key(self):
        pass


class TestLocalIdProviderDelete(TestCase):
    def test_successful_delete_returns_200(self):
        pass

    def test_non_existing_record_returns_404(self):
        pass
